@@ -238,10 +238,81 @@ def plot_headways(stats, max_headway_limit=60):
         axes[i].set_ylabel(ylabels[i])
     return fig
 
-def plot_routes_time_series(time_series):
+def agg_routes_stats(routes_stats):
     """
-    Given a stops or routes time series data frame,
-    sum each time series statistic over all stops/routes, 
+    Given ``route_stats`` which is the output of ``get_routes_stats()``,
+    return a Pandas data frame with the following columns:
+
+    - direction_id
+    - mean_daily_num_trips: the sum of the corresponding column in the
+      input across all routes
+    - min_start_time: the minimum of the corresponding column of the input
+      across all routes
+    - max_end_time: the maximum of the corresponding column of the input
+      across all routes
+    - mean_daily_duration: the sum of the corresponding column in the
+      input across all routes
+    - mean_daily_distance: the sum of the corresponding column in the
+      input across all routes  
+    - mean_daily_speed: mean_daily_distance/mean_daily_distance
+
+    If the input has no direction id, then the output won't.
+    """
+    f = routes_stats
+    if 'direction_id' in routes_stats.columns:
+        g = f.groupby('direction_id').agg({
+          'min_start_time': min, 
+          'max_end_time': max, 
+          'mean_daily_num_trips': sum,
+          'mean_daily_duration': sum,
+          'mean_daily_distance': sum,
+          }).reset_index()
+    else:
+        g = pd.DataFrame([[
+          f['min_start_time'].min(), 
+          f['max_end_time'].max(), 
+          f['mean_daily_num_trips'].sum(),
+          f['mean_daily_duration'].sum(),
+          f['mean_daily_distance'].sum(),
+          ]], 
+          columns=['min_start_time', 'max_end_time', 'mean_daily_num_trips', 
+            'mean_daily_duration', 'mean_daily_distance']
+          )
+
+    g['mean_daily_speed'] = g['mean_daily_distance'].divide(g['mean_daily_duration'])
+    return g
+
+def agg_routes_time_series(routes_time_series):
+    rts = routes_time_series
+    stats = rts.columns.levels[0].tolist()
+    split_directions = 'direction_id' in rts.columns.names
+    if split_directions:
+        # For each stat and each direction, sum across routes.
+        frames = []
+        for stat in stats:
+            f0 = rts.xs((stat, '0'), level=('statistic', 'direction_id'), 
+              axis=1).sum(axis=1)
+            f1 = rts.xs((stat, '1'), level=('statistic', 'direction_id'), 
+              axis=1).sum(axis=1)
+            f = pd.concat([f0, f1], axis=1, keys=['0', '1'])
+            frames.append(f)
+        F = pd.concat(frames, axis=1, keys=stats, names=['statistic', 
+          'direction_id'])
+        # Fix speed
+        F['mean_daily_speed'] = F['mean_daily_distance'].divide(
+          F['mean_daily_duration'])
+        result = F
+    else:
+        f = pd.concat([rts[stat].sum(axis=1) for stat in stats], axis=1, 
+          keys=stats)
+        f['mean_daily_speed'] = f['mean_daily_distance'].divide(f['mean_daily_duration'])
+        result = f
+    return result
+
+def plot_routes_time_series(routes_time_series):
+    """
+    Given a routes time series data frame,
+    sum each time series statistic over all routes, 
     plot each series statistic using MatplotLib, 
     and return the resulting figure of subplots.
 
@@ -252,47 +323,42 @@ def plot_routes_time_series(time_series):
     """
     import matplotlib.pyplot as plt
 
-    if 'route_id' not in time_series.columns.names:
+    rts = routes_time_series
+    if 'route_id' not in rts.columns.names:
         return
 
-    # Set Pandas plot style
-    pd.options.display.mpl_style = 'default'
+    # Aggregate time series
+    f = agg_routes_time_series(rts)
+
     # Reformat time periods
-    time_series.index = [t.time().strftime('%H:%M') 
-      for t in time_series.index.to_datetime()]
-    split_directions = 'direction_id' in time_series.columns.names
-    # Split time series into its component time series
-    ts_dict = {stat: time_series[stat] 
-      for stat in time_series.columns.levels[0]}
-    # For each time series sum it across its routes/stops but keep
-    # the directions separate if they already are
-    if split_directions:
-        ts_dict = {stat: f.groupby(level='direction_id', axis=1).sum()
-          for (stat, f) in ts_dict.items()}
-    else:
-        ts_dict = {stat: f.sum(axis=1)
-          for (stat, f) in ts_dict.items()}
-    # Fix speed
-    ts_dict['mean_daily_speed'] = ts_dict['mean_daily_distance'].divide(
-      ts_dict['mean_daily_duration'])
-    # Create plots  
-    names = [
-      'mean_daily_num_trip_starts', 
-      'mean_daily_num_vehicles', 
+    f.index = [t.time().strftime('%H:%M') 
+      for t in rts.index.to_datetime()]
+    
+    #split_directions = 'direction_id' in rts.columns.names
+
+    # Split time series by into its component time series by statistic type
+    # stats = rts.columns.levels[0].tolist()
+    stats = [
+      'mean_daily_num_trip_starts',
+      'mean_daily_num_vehicles',
       'mean_daily_distance',
-      'mean_daily_duration', 
+      'mean_daily_duration',
       'mean_daily_speed',
       ]
-    titles = [name.capitalize().replace('_', ' ') for name in names]
+    ts_dict = {stat: f[stat] for stat in stats}
+
+    # Create plots  
+    pd.options.display.mpl_style = 'default'
+    titles = [stat.capitalize().replace('_', ' ') for stat in stats]
     units = ['','','km','h', 'kph']
     alpha = 1
-    fig, axes = plt.subplots(nrows=len(names), ncols=1)
-    for (i, name) in enumerate(names):
-        if name == 'mean_daily_speed':
+    fig, axes = plt.subplots(nrows=len(stats), ncols=1)
+    for (i, stat) in enumerate(stats):
+        if stat == 'mean_daily_speed':
             stacked = False
         else:
             stacked = True
-        ts_dict[name].plot(ax=axes[i], alpha=alpha, 
+        ts_dict[stat].plot(ax=axes[i], alpha=alpha, 
           kind='bar', figsize=(8, 10), stacked=stacked, width=1)
         axes[i].set_title(titles[i])
         axes[i].set_ylabel(units[i])
